<?php
/*
 * np - Nekopps static functions (used to be 'npfuncs' instead of 'np')
 * Copyright (C) Nekopps all rights reserved. (http://www.nekopps.com/)
 * 2009/05/20 - 2014/09/24
 */ 

class np
{

//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
//-- files
// bool			filemake	2008-06-22 - 2011/02/09
// bool			fileappend	2014/03/10
// string		filecont	2009/06/07 - 2012/12/02
// bool			filereduce	2009/07/13
// mixed		filecache	2014/09/24
// bool			mklock		2010/02/25
// bool			unlock		2010/02/25 - 2012/05/07
//-- contents 
// array		csvread		2008-06-24 - 2012/10/30
// hash			inifile		2009/06/18 - 2014/01/12
// bool			note		2012/06/26
// array		recentuse	2011/10/07
// int			counter		2006/08/14 - 2012/10/31
//-- arrays 
// void			sortby		2012/05/11 - 2013/09/11
// array		info		2012/10/05 - 2012/10/17
// array		stripslashes_deep 2013/01/11
// array		unisubmit	2013/02/25
// void			unicharcode	2013/12/18 - 2014/01/15
// array		reqinfo		2014/03/14
//-- dates & numbers 
// string		date		2013/09/12
// string		ddmmmyyyy	2012/10/03
// int			age			2012/12/01
// array		scale		2013/09/21
//-- strings
// string		random_str	2012/03/29 - 2012/12/12
// bool			is_valid_id	2011/02/17 - 2014/03/27
// string		urlregexp	2013/08/27
// string		urlescape	2010/08/17
// string		urlfull		2012/01/20
// string		urlscript	2014/01/08
// bool, array	urlmatch	2013/02/28
// string		unibr		2009/06/29
// string		is_blank	2012/11/27
//-- XML, HTML
// string		sanitize	2007-07-04
// string		unsanitize	2011/12/07
// string		htmlpara	2011/09/01
// string		tag_contents 2009 a little faster than xml_contents? 
// mixed		xml_contents 2008-06-26, 2010/03/19
// hash			xml_recursive 2008-06-26 - 2014/04/11
//-- mails
// bool			sendmail	2011/02/19 - 2012/03/26
// bool			is_email	2007-09-05 - 2012/11/26
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function filemake($fname, $contents = null, $mode = null)
{
	if($fh = fopen($fname, "wb") and @ flock($fh, 2))
	{
		if(! is_null($contents))
			fwrite($fh, $contents); 
		fclose($fh); 
		if(isset($mode))
			if(! chmod($fname, $mode)) 
				trigger_error("Cannot change mode: $fname"); 
		return true; 
	}

	trigger_error("Cannot write to: $fname"); 
	return false; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function fileappend($fname, $contents = null)
{
	if($fh = fopen($fname, "ab") and @ flock($fh, 2))
	{
		if(! is_null($contents))
			fwrite($fh, $contents); 
		fclose($fh); 
		return true; 
	}

	trigger_error("Cannot append to: $fname"); 
	return false; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function filecont($fname)
{
	if(! is_file($fname))
	{
		trigger_error("File not found: $fname"); 
		return false; 
	}

	// this is important when you calls the same file for twice or more
	clearstatcache(); 

	$cont = ""; 
	if($fh = fopen($fname, "rb") and @ flock($fh, 1))
	{
		if($fsize = filesize($fname))
			$cont = fread($fh, $fsize); 
		fclose($fh); 
	}
	else
	{
		trigger_error("Cannot open file: $fname"); 
		return false; 
	}
	return $cont; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function filereduce($fname, $start, $end = null)
{
	$readunit = 4096; 
	if(
		is_file($fname) and 
		$fsize = filesize($fname) and 
		$fh = fopen($fname, "rb+") and 
		@ flock($fh, 2) and 
		$tmp = tmpfile()
	)
	{
		if($start > $fsize) $start = $fsize; 
		if(empty($end)) $end = $fsize; 
		fseek($fh, $start, 0); 
		while($read = fread($fh, $readunit))
		{
			$pos = ftell($fh); 
			if($pos > $end)
			{
				$read = substr($read, 0, strlen($read)-($pos-$end)); 
				fwrite($tmp, $read); 
				break; 
			}
			fwrite($tmp, $read); 
		}
		rewind($fh); 
		rewind($tmp); 
		$len = 0; 
		while($read = fread($tmp, $readunit))
		{
			$len += strlen($read); 
			fwrite($fh, $read); 
		}
		ftruncate($fh, $len); 
		fclose($fh); 
		fclose($tmp); 
	}
	else
	{
		trigger_error("Cannot prepare the working files"); 
		return false; 
	}
	return true; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function filecache($fname_or_params = null, $contents = null, $life = null)
{
	static $enabled, $basepath, $lifetime, $extension; 

	$fname = null; 
	if(is_array($fname_or_params))
	{
		$keys = explode(" ", "enabled basepath lifetime extension"); 
		while(list($key,$val) = each($fname_or_params))
			if(in_array($key, $keys))
				$$key = $val; 
			else
				trigger_error("Unsupported parameter: $key"); 
		return true; 
	}
	$fname = $fname_or_params; 
	if(! is_string($fname))
	{
		trigger_error("The filename may not be a string: $fname"); 
		return; 
	}

	if(! isset($enabled))
		$enabled = true; 
	if(! isset($basepath))
		$basepath = "."; 
	if(! isset($extension))
		$extension = "cache"; 
	if(! isset($lifetime))
		$lifetime = 60*10; 
	if(isset($life))
		$lifetime = intval($life); 

	if(! $enabled)
		return false; 
	if(! $fname)
		return false; 

	if(! is_writable($basepath))
	{
		trigger_error("Cannot write in the cache directory: $basepath"); 
		return; 
	}

	$basepath = preg_replace('|/$|', "", $basepath); 
	if($basepath)
		$fname = $basepath."/".$fname.
			($extension ? ".".$extension : ""); 

	// to delete caches, pass the 2nd arg false
	if($contents === false)
	{
		foreach(glob($fname) as $cache)
		{
			if(! unlink($cache))
				trigger_error("Failed to unlink a cache: $cache"); 
			if(file_exists($cexp = "$cache.exp"))
				if(! unlink($cexp))
					trigger_error("Failed to unlink a cache-exp: $cexp"); 
		}
		return; 
	}
	if(preg_match('/[\Q*?[]{}\E]/', $fname))
	{
		trigger_error("Invalid filename with wildcards: $fname"); 
		return; 
	}

	$fexp = "$fname.exp"; 
	if(file_exists($fexp))
		if(time() > file_get_contents($fexp))
			if(! unlink($fname) || ! unlink($fexp))
				trigger_error("Failed to unlink caches either or both of: $fname; $fexp"); 

	if(is_null($contents))
		if(file_exists($fname))
			return file_get_contents($fname); 
		else
			return false; 

	if($lifetime < 0)
		return true; 

	if(file_put_contents($fname, $contents, LOCK_EX))
		if($lifetime > 0)
			file_put_contents($fexp, time()+$lifetime, LOCK_EX); 

	if(! file_exists($fname))
	{
		trigger_error("Failed to make a cache: $fname"); 
		return false; 
	}
	return true; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function mklock($lockdir, $seconds = 180, $try = 3)
{
	if(@ is_dir($lockdir))
		if(time() > filemtime($lockdir)+$seconds)
			if(! rmdir($lockdir))
			{
				trigger_error("Cannot remove an old lockdir '$lockdir'"); 
				return false; 
			}
	for($i=0; $i<$try; $i++)
	{
		if(@ ! is_dir($lockdir))
			if(mkdir($lockdir, 0777)) 
				return true; 
		sleep(1); 
	}
	return false; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function unlock($lock)
{
	if(is_dir($lock) and ! rmdir($lock))
	{
		trigger_error("Failed to unlock: $lock"); 
		return false; 
	}
	return true; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function csvread($fname, $args = array()) 
{
	$records = array(); 
	if(0 == filesize($fname))
		return $records; 

	$fh = fopen($fname, "rb"); 
	if(! $fh) 
		return false; 
	@ fseek($fh, 0, 0); 
	@ flock($fh, LOCK_SH); 

	$sep = strval(@ $args["separator"]); 
	if(! $sep)
	{
		$line = trim(fgets($fh)); 
		rewind($fh); 
		$sep = preg_match('/[,\t]/', $line, $m) ? $m[0] : ","; 
	}

	if(! isset($args["order"]))
	{
		$line = trim(trim(fgets($fh)), $sep); 
		if(empty($line))
			return false; 
		$fields = explode($sep, $line); 
		if(@ $args["headers"])
		{ $args["order"] = $fields; }
		else
		{
			$args["order"] = range(0, -1+sizeof($fields)); 
			fseek($fh, 0, 0); 
		}
	}

	while($line = fgets($fh))
	{
		$line = trim($line); 
		if(empty($line))
			continue; 

		$item = array(); 
		$fields = explode($sep, $line); 
		for($i=0; $i<sizeof($args["order"]); $i++) 
			$item[$args["order"][$i]] = @ $fields[$i]; 
		if(@ $args["id"])
			$records[@ $item[$args["id"]]] = $item; 
		else
			$records[] = $item; 
	}
	fclose($fh); 
	return $records; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function inifile($fname, $ini = null)
{
	$namereg = "/^[_0-9A-Za-z]+$/"; 

	if(is_array($ini))
	{
		if(! ($fh = fopen($fname, "wb") and flock($fh, 2)))
		{
			trigger_error("Failed to open the inifile for writing: $fname"); 
			return; 
		}
		fseek($fh, 0, 0); 

		reset($ini); 
		while(list($name,$value) = each($ini))
		{
			if(preg_match('/^@/', $name))
				continue; 

			if(is_array($value))
				continue; 

			if(! preg_match($namereg, $name))
			{
				trigger_error("Invalid ini-property name: $name"); 
				continue; 
			}

			if(preg_match('/[\r\n]/', $value))
				$value = "\n$value"; 
			fwrite($fh, "$name = $value\n"); 
		}
		reset($ini); 
		while(list($name,$value) = each($ini))
		{
			if(preg_match('/^@/', $name))
				continue; 

			if(! is_array($value))
				continue; 

			if(! preg_match($namereg, $name))
			{
				trigger_error("Invalid ini-section name: $name"); 
				continue; 
			}

			fwrite($fh, "\n[$name]\n"); 
			while(list($n,$v) = each($value))
			{
				if(preg_match('/[\r\n]/', $v))
					$v = "\n$v"; 
				fwrite($fh, "$n = $v\n"); 
			}
		}
		fclose($fh); 
		return true; 
	}

	$ini = array(); 

	if(! file_exists($fname))
		return $ini; 
	if(! ($fh = fopen($fname, "rb") and flock($fh, 1)))
	{
		trigger_error("Failed to open the inifile for reading: $fname"); 
		return; 
	}

	$last = null; 
	$section = & $ini; 
	while($line = fgets($fh))
	{
		if(preg_match('/^[#;]/', $line))
			continue; 

		if(! preg_match('/\S/', $line))
		{
			$last = null; 
			continue; 
		}

		if(preg_match('/^\[(.*)\]$/', $line, $matchs))
		{
			$name = $matchs[1]; 
			if($name == "")
			{
				$section = & $ini; 
				continue; 
			}
			if(! preg_match($namereg, $name))
			{
				trigger_error("Invalid ini-section name: $name"); 
				continue; 
			}
			if(isset($ini[$name]))
				trigger_error("Ini-property overwriting with a section: $name"); 
			if(! isset($ini["@sections"]))
				$ini["@sections"] = array(); 
			$ini["@sections"][] = $name; 
			$ini[$name] = array(); 
			$section = & $ini[$name]; 
			continue; 
		}

		if(false === strpos($line, "="))
		{
			if($last)
				$section[$last] .= $line; 
			continue; 
		}

		$line = trim($line); 
		$pair = explode("=", $line, 2); 
		$name = trim($pair[0]); 
		if(empty($name))
			continue; 
		if(! preg_match($namereg, $name))
		{
			trigger_error("Invalid ini-property name: $name"); 
			continue; 
		}
		if(isset($section[$name]))
			trigger_error("Ini-property overwriting with a scalar: $name"); 

		$val = isset($pair[1]) ? trim($pair[1]) : ""; 
		$val = preg_replace('/\{\$([^\}]*)\}/e', '$section["$1"]', $val); 
		$val = preg_replace('/\{\&([^\}]*)\}/e', 'eval("return $1;")', $val); 
		$section[$name] = $val; 
		$last = $name; 
	}
	fclose($fh); 

	return $ini; 
}
# -- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
static function note($msg, $logdir, $fname = "note", $period = 4, $fnum = 5)
{
	if(! is_dir($logdir) or ! is_writable($logdir))
	{
		trigger_error("Directory not exists or not writable: $logdir"); 
		return; 
	}

	// determing the logfile name
	do
	{ 
		$t = localtime();

		$date  = sprintf("%04d", $t[5]+1900); 
		if($period == 5) break; 

		$date .= sprintf("%02d", $t[4]+1); 
		if($period == 4) break; 

		$date .= sprintf("%02d", $t[3]); 
		if($period == 3) break; 
		if($period == 6)
		{
			$weeks = explode(" ", "sun mon tue wed thu fri sat"); 
			$date .= $weeks[$t[6]]; 
			break; 
		}

		$date .= sprintf("%02d", $t[2]); 
		if($period == 2) break; 

		$date .= sprintf("%02d", $t[1]); 
		if($period == 1) break; 

		$date .= sprintf("%02d", $t[0]); 
		if($period == 0) break; 
	}
	while(0); 
	$log = "$logdir/$fname-$date.log"; 

	// removing the oldest logfile 
	if(! file_exists($log))
	{
		$loglist = glob("$logdir/$fname-*.log"); 
		if(count($loglist) >= $fnum)
		{
			usort($loglist, create_function('$a,$b', '
				$am = filemtime($a); 
				$bm = filemtime($b); 
				return $am == $bm ? 0 : ($am < $bm ? -1 : 1); 
			')); 
			$oldest = $loglist[0]; 
			if(! unlink($oldest))
				trigger_error("Cannot unlink an old logfile: $oldest"); 
		}
	}

	if($fh = fopen($log, "ab") and @ flock($fh, 1))
	{
		fwrite($fh, $msg); 
		fwrite($fh, "\n"); 
		fclose($fh); 
	}
	else
	{
		trigger_error("Cannot open a logfile: $log"); 
		return false; 
	}

	return true; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
static function recentuse($dat, $second = null)
{
	if($recent = is_array($second) ? $second : null)
	{
		$fh = fopen($dat, "wb"); 
		if(! $fh)
		{
			trigger_error("Cannot write-open: $dat"); 
			return; 
		}
		@ flock($fh, 2); 
		while(list($uni, $times) = each($recent))
		{
			if(is_scalar($times))
				$times = array($times); 
			foreach($times as $time)
				fwrite($fh, "$time\t$uni\n"); 
		}
		fclose($fh); 
	}
	else
	{
		$recent = array(); 
		$while = $second ? intval($second) : 60*10; 
		if(file_exists($dat))
			foreach(file($dat) as $line)
			{
				list($time, $uni) = explode("\t", trim($line)); 
				if(time()-$time <= $while)
					if(isset($recent[$uni]))
						$recent[$uni][] = $time; 
					else
						$recent[$uni] = array($time); 
			}
	}
	return $recent; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
static function counter($dat, $add = true)
{
	if(! file_exists($dat))
		if(! np::filemake($dat, 0))
			return; 
	$add = intval($add); 

	if(! ($fh = fopen($dat, "rb") and flock($fh, 1)))
	{
		trigger_error("Failed to open the counter file for reading: $dat"); 
		return; 
	}
	$count = fgets($fh); 
	fclose($fh); 

	$count = intval(trim($count)); 
	if(! $add)
		return $count; 
	$count += $add; 

	if(! ($fh = fopen($dat, "rb+") and flock($fh, 2)))
	{
		trigger_error("Failed to open the counter file for read-writing: $dat"); 
		return; 
	}
	fseek($fh, 0, 0); 
	fputs($fh, $count); 
	ftruncate($fh, strlen($count)); 
	fclose($fh); 

	return $count; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
static function sortby(& $tosort, $by = null)
{
	if(! is_array($tosort) or empty($by))
	{
		trigger_error("Miss arguments for sortby()"); 
		return; 
	}

	$by = preg_split('/\s*,\s*/', $by); 
	$GLOBALS["_by"] = $by; 

	if(! function_exists("_inner_sortby"))
	{
		function _inner_sortby($a, $b)
		{
			global $_by; 
			foreach($_by as $by)
			{
				$fst = substr($by, 0, 1); 
				$descent = false; 
				$asstr = false; 
				if($fst == "+" or $fst == "-")
				{
					$descent = $fst == "-"; 
					$by = substr($by, 1, strlen($by)-1); 
				}
				if(preg_match('/(\:string)$/', $by, $m))
				{
					$asstr = true; 
					$by = substr($by, 0, strlen($by)-strlen($m[1])); 
				}

				$av = $a[$by]; 
				$bv = $b[$by]; 
				if($asstr)
					$ret = $descent ? strcmp($bv, $av) : strcmp($av, $bv); 
				else
					$ret = $av == $bv ? 0 : (($descent ? $bv > $av : $av > $bv) ? 1 : -1); 
				if($ret != 0)
					return $ret; 
			}
			return 0; 
		}
	}
	return usort($tosort, "_inner_sortby"); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function info($key_or_pairs = null, $val_or_include = null)
{
	static $info = array(); 
	$key = $pairs = $key_or_pairs; 
	if(is_null($key))
		return $info;  
	$val = $file = $val_or_include; 
	if(is_scalar($key))
	{
		$info[$key] = is_null($val) ? $key : $val; 
		return $info[$key]; 
	}
	foreach($pairs as $k => $v) 
		if(isset($info[$k]))
			$info[$k] = $v;
	if($file)
		include $file; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function stripslashes_deep($vals)
{
	// make the magic-quotes no effect
	if(is_scalar($vals))
		$vals = stripslashes($vals); 
	else
		foreach(array_keys($vals) as $key)
		{
			$val = $vals[$key]; 
			$vals[$key] = is_scalar($val) ? stripslashes($val) : np::stripslashes_deep($val); 
		}
	return $vals; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function unisubmit($in, $prefix)
{
	// this is the code that copes with the diffrence of 
	// data-sending way of 'type="image"' across some browsers. 
	foreach(array_keys($in) as $key)
		if(0 === strpos($key, $prefix))
			if(preg_match('/[\._]x$/', $key))
				if($key = substr($key, 0, strlen($key)-2))
					if(@ ! $in[$key])
						$in[$key] = 1; 
	return $in; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function unicharcode(& $in, $code = null)
{
	$code = $code ? $code : "UTF-8"; 
	$order = "UTF-8,SJIS,EUC-JP,JIS,ISO-2022-JP"; 
	if(empty($in))
		return; 

	if(is_scalar($in))
	{
		if($v = trim($in))
			if($enc = mb_detect_encoding($v, $order))
				if($enc != $code)
					$in = mb_convert_encoding($v, $code, $enc); 
		return; 
	}

	foreach(array_keys($in) as $k)
		if(is_array($in[$k]))
			np::unicharcode($in[$k], $code); 
		else if($v = trim(@ $in[$k]))
			if($enc = mb_detect_encoding($v, $order))
				if($enc != $code)
					$in[$k] = mb_convert_encoding($v, $code, $enc); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function reqinfo($gethost = true)
{
	$req = array(
		"method"	=> $_SERVER["REQUEST_METHOD"], 
		"uri"		=> $_SERVER["REQUEST_URI"], 
		"referer"	=> @ $_SERVER["HTTP_REFERER"], 
		"user"		=> @ $_SERVER["REMOTE_USER"], 
		"host"		=> null, 
		"addr"		=> $_SERVER["REMOTE_ADDR"], 
		"host"		=> gethostbyaddr($_SERVER["REMOTE_ADDR"]), 
		"agent"		=> @ $_SERVER["HTTP_USER_AGENT"], 
	); 
	$req["url"] = $req["uri"]; 
	if($gethost)
		$req["host"] = gethostbyaddr($req["addr"]); 
	return $req; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function date($format, $time = null, $wdays = null)
{
	if(is_null($time))
		$time = time(); 
	if(empty($wdays))
		return date($format, $time); 

	$fulls = explode(" ", "Sunday Monday Tuesday Wednesday Thursday Friday Saturday"); 
	if(! is_array($wdays))
		$wdays = preg_split('/\s+/', $wdays); 
	if(count($wdays) != 7)
	{
		trigger_error("Weekdays must be 7"); 
		return; 
	}

	$format = str_replace("w", "l", $format); 
	$ret = date($format, $time); 
	return str_replace($fulls, $wdays, $ret); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function ddmmmyyyy($at = null)
{
	$at = $at ? $at : date("Ymd"); 
	list($y,$m,$d) = array(substr($at,0,4), substr($at,4,2), substr($at,6,2)); 
	return date("d M Y", mktime(0,0,0,intval($m),intval($d),intval($y))); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function age($ymd, $m = null, $d = null)
{
	if($ymd && $m && $d)
		$y = $ymd; 
	else
	{
		if(preg_match('/(\D)/', $ymd, $matchs))
			list($y,$m,$d) = explode($matchs[1], $ymd, 3); 
		else
			return; 
	}

	list($cy,$cm,$cd) = explode("/", date("Y/m/d")); 
	foreach(explode(" ", "y m d cy cm cd") as $var)
		$$var = intval($$var); 

	if(! checkdate($m,$d,$y))
		return false; 

	$age = $cy - $y; 
	if($m > $cm or $m == $cm && $d > $cd)
		$age -= 1; 

	return $age; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function scale($w, $h, $pw, $ph, $enlarge = false)
{
	$dw = $pw - $w; 
	$dh = $ph - $h; 
	if($dw > 0 or $dh > 0)
	{
		if($dw > $dh)
			$h = round($ph * ($w / $pw));
		else
			$w = round($pw * ($h / $ph)); 
	}
	else if(($dw < 0 or $dh < 0) and $enlarge)
	{
		$dw = abs($dw); 
		$dh = abs($dh); 
		if($dw < $dh)
			$h = round($ph * ($w / $pw));
		else
			$w = round($pw * ($h / $ph)); 
	}
	else
	{ 
		$w = $pw; 
		$h = $ph; 
	}
	return array($w, $h); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function random_str($len = 8, $chars = null)
{
	$str = array(); 
	if(empty($chars))
		$chars = array_merge(range(0,9),range('A','Z'),range('a','z'),array(".","_")); 
	//return substr(str_shuffle(implode("", $chars)), 0, $len); 
	for($i=0; $i<$len; $i++)
	{
		$idx = rand(0, count($chars)-1); 
		$str[] = $chars[$idx]; 
	}
	return implode("", $str); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function is_valid_id($id)
{ return preg_match('/^[_A-Za-z\d]{2,}$/', $id); }
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function urlregexp()
{
	# $printable = ' !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'; 
	$canbeused = '!#$%&*,-./0123456789:;=?@ABCDEFGHIJKLMNOPQRSTUVWXYZ^_abcdefghijklmnopqrstuvwxyz|~'; 
	return '[0-9a-z]+\:\/\/['.preg_quote($canbeused, "/").']+'; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function urlescape($val)
{
	// RFC3986 base encoding
	$val = preg_replace('/([^A-Za-z0-9_.~-])/e', "sprintf('%%%02X', ord('\$1'))", $val); 
	return $val; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function urlfull($env = null)
{
	if(empty($env))
		$env = $_SERVER; 

	return 
		"http".
		(preg_match('/https/i', $env["SERVER_PROTOCOL"]) ? "s" : "").
		"://".$env["SERVER_NAME"].$env["REQUEST_URI"]; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function urlscript($env = null)
{
	if(empty($env))
		$env = $_SERVER; 

	return 
		"http".
		(preg_match('/https/i', $env["SERVER_PROTOCOL"]) ? "s" : "").
		"://".$env["SERVER_NAME"].$env["SCRIPT_NAME"]; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function urlmatch($string, $capture = false)
{ 
	$re = np::urlregexp(); 
	if($capture)
	{
		preg_match_all("/$re/", $string, $matchs); 
		return $matchs[0]; 
	}	
	return preg_match('/^'.$re.'$/', $string); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function unibr($str, $br = "\012")
{
	$str = str_replace("\015\012", $br, $str); 
	$str = str_replace("\015", $br, $str); 
	return $str; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function is_blank($text, $as_html = false)
{
	if($as_html)
	{
		$text = preg_replace('/<\s*\/?\s*[^>]*>/', "", $text); 
		$text = preg_replace('/\&nbsp;/', " ", $text); 
	}
	$text = preg_replace('/\xe3\x80\x80/', " ", $text); // a space in utf8
	$text = preg_replace('/\s+/', "", $text); 
	return empty($text); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function sanitize($str)
{ return htmlspecialchars($str); } 
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function unsanitize($str)
{
	if(empty($str))
		return $str; 
	$from = array("&lt;", "&gt;", "&quot;", "&#39;", "&amp;"); 
	$to = array("<", ">", '"', "'", "&"); 
	return str_replace($from, $to, $str); 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function htmlpara($text)
{
	static $lf = "\012"; 

	$text = trim($text); 
	if(empty($text) || preg_match('/<p.*?>/', $text))
		return $text; 

	$text = np::unibr($text); 
	$paras = explode("$lf$lf", $text); 
	for($i=0; $i<count($paras); $i++)
	{
		$paras[$i] = str_replace($lf, "<br />", $paras[$i]); 
		$paras[$i] = empty($paras[$i]) ? "<br /><br />" : "<p>".$paras[$i]."</p>"; 
	}
	return implode($lf, $paras).$lf; 
}
// -- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
static function tag_contents($doc, $tag)
{
	$cont = ""; 
	if(preg_match("|<\\s*{$tag}[^>]*>\\s*|is", $doc, $matchs, PREG_OFFSET_CAPTURE))
	{
		$start = $matchs[0][1] + strlen($matchs[0][0]); 
		if(preg_match("|<\\s*/\\s*{$tag}[^>]*>|is", $doc, $matchs, PREG_OFFSET_CAPTURE, $start))
			$end = $matchs[0][1]; 
	}
	if(isset($start) && isset($end))
		$cont = substr($doc, $start, $end-$start); 
	return $cont; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function xml_contents($xmldata, $tags = null)
{
	$tagre = '@<\s*(__TAG__)[^>]*(?:/\s*>|>\s*(.*?)<\s*/\s*\1\s*>)@is'; 
	$ret = array(); 
	if(isset($tags))
	{
		if(! is_array($tags)) settype($tags, "array"); 
		foreach($tags as $tag)
		{ 
			$re = str_replace("__TAG__", $tag, $tagre); 
			if(preg_match($re, $xmldata, $matchs))
				$ret[$tag] = $matchs[2]; 
		}
	}
	else
	{
		$re = str_replace("__TAG__", '[\w\:]+', $tagre); 
		preg_match_all($re, $xmldata, $matchs); 
		for($i=0; isset($matchs[1][$i]); $i++)
		{
			$tag = $matchs[1][$i]; 
			$val = @ $matchs[2][$i]; 
			if(isset($ret[$tag]))
			{
				if(! is_array($ret[$tag])) settype($ret[$tag], "array"); 
				$ret[$tag][] = $val; 
			}
			else
				$ret[$tag] = $val; 
		}
	}
	return count($ret) ? $ret : $xmldata; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function xml_recursive($data) 
{
	if(is_scalar($data))
	{
		$data = np::xml_contents($data); 
		if(is_scalar($data)) 
			return $data; 
	}
	while(list($key,$val) = each($data))
	{
		if(is_scalar($val))
			$val = np::xml_contents($val); 
		if(is_array($val))
			$val = np::xml_recursive($val); 
		$data[$key] = $val; 
	}
	reset($data); // 2010/05/12 (very important)
	return $data; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function sendmail($header_and_body, $language = "ja")
{
	$detect_order = "UTF-8, SJIS, EUC-JP, JIS"; 

	if(! function_exists("mb_send_mail"))
	{
		trigger_error("static function mb_send_mail() doesn't exists"); 
		return; 
	}

	$both = np::unibr($header_and_body); 
	@ list($header, $body) = explode("\n\n", $both, 2); 
	if(! isset($body))
	{
		trigger_error("Both header and body needed"); 
		return; 
	}
	$to = ""; 
	$subject = ""; 
	$add = array(); 
	$header = trim($header); 
	foreach(explode("\n", $header) as $h)
	{
		if(preg_match('/^\s/', $h))
		{
			$add[] = $h; 
			continue; 
		}
		if(! preg_match('/^([^\:]+)\:(.*)/', $h, $m))
		{
			trigger_error("Invalid header: $h"); 
			return; 
		}
		$name = trim(strtolower($m[1])); 
		$value = trim($m[2]); 
		switch($name)
		{
			case "subject": 
				$subject = $value; 
				break; 
			case "to": 
				$to = $value; 
				break; 
			default: 
				$add[] = $h; 
		}
	}
	$add = implode("\n", $add); 
	if(empty($to))
	{
		trigger_error("No to-address applied"); 
		return; 
	}
	if(empty($subject))
	{
		trigger_error("No subject applied"); 
		return; 
	}

	$charcode = mb_detect_encoding($body, $detect_order); 
	$lan = mb_language(); 
	$enc = mb_internal_encoding(); 
	mb_language($language); 
	mb_internal_encoding($charcode); 
	$ret = mb_send_mail($to, $subject, $body, $add); 
	mb_language($lan); 
	mb_internal_encoding($enc); 
	if(! $ret)
		return false; 
	return true; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
static function is_email($string, $count = false)
{ 
	$re = '[-_0-9A-Za-z\.]+\@[-_0-9A-Za-z]+(?:\.[-_0-9A-Za-z]+)+'; 
	if($count)
	{
		preg_match_all("/$re/", $string, $matchs); 
		return $matchs[0]; 
	}	
	return preg_match('/^'.$re.'$/', $string); 
}

} // end of class

?>
