<?php
/*
 * npforms.class.inc scripted by Nekopps 
 * Copyright (C) Nekopps all rights reserved.
 * 2011/02/06 - 2012/01/25
 */ 

class npforms
{

//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
// string|array		item 2008-10-06 - 2012/01/25
// string|array		item_from_xmlitem 2011/02/04 - 2012/01/24
// struct			read 2011/02/06 - 2011/07/26
// void				copy 2011/02/06 - 2011/02/19
// string			simple 2011/07/25
// array			check 2011/07/25
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function item($type, $name = "", $values = null, $curval = null, $attr = null)
{
	// 2011/02/01 call the brother function below
	if(func_num_args() == 1)
		if(is_array($type))
			return npforms::item_from_xmlitem($type); 

	// make name and attributes
	$name = " name=\"$name\""; 
	if(! isset($attr)) 
		$attr = ""; 
	if(is_array($attr))
	{
		foreach(array_keys($attr) as $key)
			$attr[$key] = " $key=\"".htmlspecialchars($attr[$key]).'"'; 
		$attr = implode("", $attr); 
	}
	elseif(strlen($attr) and ! preg_match('/^\s/', $attr))
		$attr = " ".$attr; 

	// single value types 
	if(in_array($type, array("text", "password", "hidden", "textarea")))
	{
		if(! is_scalar($curval)) 
			$curval = serialize($curval); 

		// type of textarea 
		if("textarea" == $type)
		{
			$value = htmlspecialchars($curval); 
			return "<textarea$name$attr>$value</textarea>"; 
		}

		$type = " type=\"$type\""; 
		$value = " value=\"".htmlspecialchars($curval).'"'; 
		return "<input$type$name$value$attr />"; 
	}

	if(! is_array($values)) 
		$values = preg_split('/\s+/', $values); 
	if(! is_array($curval)) 
		$curval = preg_split('/\s+/', $curval); 
	$pairs = array(); 
	foreach($values as $val)
	{
		$label = null; 
		if(false !== ($pos = strpos($val, ":")))
		{
			list($label, $val) = explode(":", $val, 2); 
			$label = trim($label); 
			$val = trim($val); 
		}
		$pairs[] = array($label, $val); 
	}

	// type of select 
	if("select" == $type)
	{		
		$options = array(); 
		foreach($pairs as $pair)
		{
			list($label, $val) = $pair; 
			$selected = in_array($val, $curval) ? ' selected="selected"' : ""; 
			if(isset($label))
			{
				$label = htmlspecialchars($label); 
				$value = " value=\"".htmlspecialchars($val).'"'; 
			}
			else
			{ 
				$label = htmlspecialchars($val); 
				$value = ""; 
			}
			$options[] = "<option$value$selected>$label</option>"; 
		}
		return "<select$name$attr>\n".join("\n", $options)."\n</select>"; 
	}

	// type of checkbox and radiobox
	if("checkbox" == $type or "radio" == $type)
	{
		$boxes = array(); 
		$type = " type=\"$type\""; 
		$uniq = uniqid("boxes-"); 
		$num = 1; 
		foreach($pairs as $pair)
		{
			list($label, $val) = $pair; 
			if(empty($label)) 
				$label = $val; 
			$value = " value=\"".htmlspecialchars($val).'"'; 
			$checked = in_array($val, $curval) ? ' checked="checked"' : ""; 
			$id = " id=\"$uniq-$num\""; 
			$label = "<label for=\"$uniq-$num\">$label</label>"; 
			$boxes[] = "<input$type$name$value$checked$id$attr />$label"; 
			$num++; 
		}
		return count($boxes) == 1 ? $boxes[0] : $boxes; 
	}

	trigger_error("Unknown item type '$type'"); 
	return false; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function item_from_xmlitem($item)
{
	$tags = explode(" ", "type name value option multiple need errmsg errchk caption prefix suffix note"); 
	foreach($tags as $tag)
		$$tag = isset($item[$tag]) ? $item[$tag] : null; 
	if(is_null($type))
		$type = "text"; 
	if(is_null($name))
		$name = @ $item["id"]; 
	if($multiple)
		$name .= "[]"; 
	if(is_null($value))
		$value = ""; 
	foreach($tags as $tag)
		unset($item[$tag]); 

	$attr = array(); 
	foreach($item as $key => $val)
		if(preg_match('/^[a-z]/i', $key))
			if(is_scalar($val)) 
				$attr[$key] = $val; 
	if($type == "select" && $multiple)
		$attr["multiple"] = "multiple"; 

	$input = null; 
	switch($type)
	{
		case "select": 
		case "checkbox": 
		case "radio"
		: 
			$values = array(); 
			if(! isset($option))
				$option = ""; 
			if(! is_array($option) or ! isset($option[0]))
				$option = array($option); 
			foreach($option as $opt)
				if(is_scalar($opt))
					$values[] = $opt; 
				else 
					$values[] = $opt["-content"].":".$opt["@value"]; 
			$input = npforms::item($type, $name, $values, $value, $attr); 
			break; 

		case "shown"
		: 
			$input = npforms::item("hidden", $name, $value, null, $attr); 
			$input .= $value; 
			break; 

		case "display"
		:
			$input = $value; 
			break; 

		case "file": 
		case "text": 
		case "password": 
		case "hidden": 
		case "textarea": 
		default
		: 
			$input = npforms::item($type, $name, $value, null, $attr); 
			break; 
	}
	return $input; 
}
//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function read($xmlfile)
{
	if(! class_exists("npxml"))
	{
		trigger_error("Class 'npxml' is not loaded"); 
		return; 
	}

	$xml = new npxml($xmlfile); 
	$xml->parse(); 
	$fitems = $xml->tree["formitems"]; 

	$groups = @ $fitems["group"]; 
	if(! isset($groups))
		$groups = array(array("item" => @ $fitems["item"])); 
	if(! isset($groups[0]))
		$groups = array($groups); 
	for($i=0; $i<count($groups); $i++)
		if(! isset($groups[$i]["item"][0]))
			$groups[$i]["item"] = array($groups[$i]["item"]); 
	$fitems["group"] = $groups; 

	$fitems["items"] = array(); 
	for($g=0; $g<count($fitems["group"]); $g++)
	{
		$name = @ $fitems["group"][$g]["name"]; 
		if($name)
			$fitems["#$name"] = & $fitems["group"][$g]; 
		for($i=0; $i<count($fitems["group"][$g]["item"]); $i++)
		{
			$fitems["items"][] = & $fitems["group"][$g]["item"][$i]; 
			$name = @ $fitems["group"][$g]["item"][$i]["name"]; 
			if($name)
				$fitems["#$name"] = & $fitems["group"][$g]["item"][$i]; 
		}
	}

	return $fitems; 
}

//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function copy($fitems, $from, $into, $exchange = null)
{
	for($g=0; $g<count($fitems["group"]); $g++)
		for($i=0; $i<count($fitems["group"][$g]["item"]); $i++)
		{
			$item = & $fitems["group"][$g]["item"][$i]; 
			$val = @ $from[$item["name"]]; 
			if(@ $item["multiple"] && $exchange)
				if(is_array($val))
					$val = implode(" ", $val); 
				else
					$val = explode(" ", $val); 
			if(is_null($into))
				$item["value"] = $val; 
			else
				$into[$item["name"]] = $val; 
		}
}

//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function simple($fitems)
{
	// often, this may be used as a template 

	ob_start(); 
	?>
<?php foreach($fitems["group"] as $group){ ?>
<?php if(@ $group["caption"]){ ?>
<fieldset style="margin-bottom: 1em; ">
<legend><?=$group["caption"]?></legend>
<?php } ?> 
<?php foreach($group["item"] as $item){ ?>
<?php 
	if($item["type"] == "none") 
		continue; 
	$input = npforms::item($item); 
	$input = is_array($input) ? implode("\n", $input) : $input; 
	if($item["type"] == "hidden")
	{
		echo "$input\n"; 
		continue; 
	}
?>
<p>
<?=@$item["need"] ? "<tt>*</tt>" : ""?><?=@$item["caption"]?><tt>:</tt><br />
<?=@$item["prefix"]?><?=$input?><?=@$item["suffix"]?> 
<?=@$item["note"]?> 
</p>
<?php } ?>
<?php if(@ $group["caption"]){ ?>
</fieldset>
<?php } ?> 
<?php } ?>
<?php 
	return ob_get_clean(); 
}

//-- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----
function check($fitems)
{
	$errmsg = @ $fitems["errmsg"]; 
	$errmsg = $errmsg ? $errmsg : "Please input the ${caption} correctly"; 
	$msgs = array(); 
	for($g=0; $g<count($fitems["group"]); $g++)
		for($i=0; $i<count($fitems["group"][$g]["item"]); $i++)
		{
			$item = $fitems["group"][$g]["item"][$i]; 
			$chk = @ $item["errchk"]; 
			if(@ $item["need"] or $chk)
			{
				if(empty($chk))
					$chk = '/\S/'; 
				if(is_scalar($chk))
					$chk = array("@type" => "regexp", "-content" => $chk); 
				$passed = true; 
				switch($chk["@type"])
				{
					case "phpcode": 
						$value = $item["value"]; 
						$passed = eval($chk["-content"]); 
						break; 
					case "regexp": 
						$passed = preg_match($chk["-content"], $item["value"]); 
						break; 
				}
				if(! $passed)
				{
					$msg = @ $item["errmsg"]; 
					$msg = $msg ? $msg : $errmsg; 
					$msg = preg_replace('/\$\{(.+?)\}/e', '@$item[\1]', $msg); 
					$msgs[] = $msg; 
				}
			}
		}
	return $msgs; 
}

} // end of class npforms

?>
